@@ -119,7 +119,11 @@ Read* OverlapAnalysis::merge(Read* r1, Read* r2, OverlapResult ov) {
     delete rr2;
 
     string name = *(r1->mName) + " merged_" + to_string(len1) + "_" + to_string(len2);
-    Read* mergedRead = new Read(new string(name), new string(mergedSeq), new string(*r1->mStrand), new string(mergedQual));
+    string strand = *(r1->mStrand);
+    if (strand != "+") {
+      strand = strand + " merged_" + to_string(len1) + "_" + to_string(len2);
+    }
+    Read* mergedRead = new Read(new string(name), new string(mergedSeq), new string(strand), new string(mergedQual));
 
     return mergedRead;
 }
@@ -142,4 +146,4 @@ bool OverlapAnalysis::test(){
     mergedRead->print();
 
     return ov.overlapped && ov.offset == 10 && ov.overlap_len == 79 && ov.diff == 1;
-}
\ No newline at end of file
+}
