@@ -13,11 +13,9 @@
 
 SingleEndProcessor::SingleEndProcessor(Options* opt){
     mOptions = opt;
-    mProduceFinished = false;
+    mReaderFinished = false;
     mFinishedThreads = 0;
     mFilter = new Filter(opt);
-    mOutStream = NULL;
-    mZipFile = NULL;
     mUmiProcessor = new UmiProcessor(opt);
     mLeftWriter =  NULL;
     mFailedWriter = NULL;
@@ -26,6 +24,11 @@ SingleEndProcessor::SingleEndProcessor(Options* opt){
     if(mOptions->duplicate.enabled) {
         mDuplicate = new Duplicate(mOptions);
     }
+
+    mPackReadCounter = 0;
+    mPackProcessedCounter = 0;
+
+    mReadPool = new ReadPool(mOptions);
 }
 
 SingleEndProcessor::~SingleEndProcessor() {
@@ -34,6 +37,10 @@ SingleEndProcessor::~SingleEndProcessor() {
         delete mDuplicate;
         mDuplicate = NULL;
     }
+    if(mReadPool) {
+        delete mReadPool;
+        mReadPool = NULL;
+    }
 }
 
 void SingleEndProcessor::initOutput() {
@@ -68,30 +75,31 @@ bool SingleEndProcessor::process(){
     if(!mOptions->split.enabled)
         initOutput();
 
-    initPackRepository();
-    std::thread producer(std::bind(&SingleEndProcessor::producerTask, this));
+    mInputLists = new SingleProducerSingleConsumerList<ReadPack*>*[mOptions->thread];
 
-    //TODO: get the correct cycles
-    int cycle = 151;
     ThreadConfig** configs = new ThreadConfig*[mOptions->thread];
     for(int t=0; t<mOptions->thread; t++){
+        mInputLists[t] = new SingleProducerSingleConsumerList<ReadPack*>();
         configs[t] = new ThreadConfig(mOptions, t, false);
+        configs[t]->setInputList(mInputLists[t]);
         initConfig(configs[t]);
     }
 
+    std::thread readerThread(std::bind(&SingleEndProcessor::readerTask, this));
+
     std::thread** threads = new thread*[mOptions->thread];
     for(int t=0; t<mOptions->thread; t++){
-        threads[t] = new std::thread(std::bind(&SingleEndProcessor::consumerTask, this, configs[t]));
+        threads[t] = new std::thread(std::bind(&SingleEndProcessor::processorTask, this, configs[t]));
     }
 
     std::thread* leftWriterThread = NULL;
     std::thread* failedWriterThread = NULL;
     if(mLeftWriter)
-        leftWriterThread = new std::thread(std::bind(&SingleEndProcessor::writeTask, this, mLeftWriter));
+        leftWriterThread = new std::thread(std::bind(&SingleEndProcessor::writerTask, this, mLeftWriter));
     if(mFailedWriter)
-        failedWriterThread = new std::thread(std::bind(&SingleEndProcessor::writeTask, this, mFailedWriter));
+        failedWriterThread = new std::thread(std::bind(&SingleEndProcessor::writerTask, this, mFailedWriter));
 
-    producer.join();
+    readerThread.join();
     for(int t=0; t<mOptions->thread; t++){
         threads[t]->join();
     }
@@ -135,28 +143,21 @@ bool SingleEndProcessor::process(){
     cerr << "Filtering result:"<<endl;
     finalFilterResult->print();
 
-    int* dupHist = NULL;
-    double* dupMeanTlen = NULL;
-    double* dupMeanGC = NULL;
     double dupRate = 0.0;
     if(mOptions->duplicate.enabled) {
-        dupHist = new int[mOptions->duplicate.histSize];
-        memset(dupHist, 0, sizeof(int) * mOptions->duplicate.histSize);
-        dupMeanGC = new double[mOptions->duplicate.histSize];
-        memset(dupMeanGC, 0, sizeof(double) * mOptions->duplicate.histSize);
-        dupRate = mDuplicate->statAll(dupHist, dupMeanGC, mOptions->duplicate.histSize);
+        dupRate = mDuplicate->getDupRate();
         cerr << endl;
         cerr << "Duplication rate (may be overestimated since this is SE data): " << dupRate * 100.0 << "%" << endl;
     }
 
     // make JSON report
     JsonReporter jr(mOptions);
-    jr.setDupHist(dupHist, dupMeanGC, dupRate);
+    jr.setDup(dupRate);
     jr.report(finalFilterResult, finalPreStats, finalPostStats);
 
     // make HTML report
     HtmlReporter hr(mOptions);
-    hr.setDupHist(dupHist, dupMeanGC, dupRate);
+    hr.setDup(dupRate);
     hr.report(finalFilterResult, finalPreStats, finalPostStats);
 
     // clean up
@@ -171,11 +172,6 @@ bool SingleEndProcessor::process(){
     delete finalPostStats;
     delete finalFilterResult;
 
-    if(mOptions->duplicate.enabled) {
-        delete[] dupHist;
-        delete[] dupMeanGC;
-    }
-
     delete[] threads;
     delete[] configs;
 
@@ -190,9 +186,17 @@ bool SingleEndProcessor::process(){
     return true;
 }
 
+void SingleEndProcessor::recycleToPool(int tid, Read* r) {
+    // failed to recycle, then delete it
+    if(!mReadPool->input(tid, r))
+        delete r;
+}
+
 bool SingleEndProcessor::processSingleEnd(ReadPack* pack, ThreadConfig* config){
-    string outstr;
-    string failedOut;
+    string* outstr = new string();
+    string* failedOut = new string();
+    int tid = config->getThreadId();
+
     int readPassed = 0;
     for(int p=0;p<pack->count;p++){
 
@@ -203,12 +207,16 @@ bool SingleEndProcessor::processSingleEnd(ReadPack* pack, ThreadConfig* config){
         config->getPreStats1()->statRead(or1);
 
         // handling the duplication profiling
-        if(mDuplicate)
-            mDuplicate->statRead(or1);
+        bool dedupOut = false;
+        if(mDuplicate) {
+            bool isDup = mDuplicate->checkRead(or1);
+            if(mOptions->duplicate.dedup && isDup)
+                dedupOut = true;
+        }
 
         // filter by index
         if(mOptions->indexFilter.enabled && mFilter->filterByIndex(or1)) {
-            delete or1;
+            recycleToPool(tid, or1);
             continue;
         }
 
@@ -254,26 +262,26 @@ bool SingleEndProcessor::processSingleEnd(ReadPack* pack, ThreadConfig* config){
 
         config->addFilterResult(result, 1);
 
-        if( r1 != NULL &&  result == PASS_FILTER) {
-            outstr += r1->toString();
+        if(!dedupOut) {
+            if( r1 != NULL &&  result == PASS_FILTER) {
+                r1->appendToString(outstr);
 
-            // stats the read after filtering
-            config->getPostStats1()->statRead(r1);
-            readPassed++;
-        } else if(mFailedWriter) {
-            failedOut += or1->toStringWithTag(FAILED_TYPES[result]);
+                // stats the read after filtering
+                config->getPostStats1()->statRead(r1);
+                readPassed++;
+            } else if(mFailedWriter) {
+                or1->appendToStringWithTag(failedOut, FAILED_TYPES[result]);
+            }
         }
 
-        delete or1;
+        recycleToPool(tid, or1);
         // if no trimming applied, r1 should be identical to or1
         if(r1 != or1 && r1 != NULL)
-            delete r1;
+            recycleToPool(tid, r1);
     }
-    // if splitting output, then no lock is need since different threads write different files
-    if(!mOptions->split.enabled)
-        mOutputMtx.lock();
+
     if(mOptions->outputToSTDOUT) {
-        fwrite(outstr.c_str(), 1, outstr.length(), stdout);
+        fwrite(outstr->c_str(), 1, outstr->length(), stdout);
     } else if(mOptions->split.enabled) {
         // split output by each worker thread
         if(!mOptions->out1.empty())
@@ -281,96 +289,34 @@ bool SingleEndProcessor::processSingleEnd(ReadPack* pack, ThreadConfig* config){
     } 
 
     if(mLeftWriter) {
-        char* ldata = new char[outstr.size()];
-        memcpy(ldata, outstr.c_str(), outstr.size());
-        mLeftWriter->input(ldata, outstr.size());
+        mLeftWriter->input(tid, outstr);
+        outstr = NULL;
     }
-    if(mFailedWriter && !failedOut.empty()) {
+    if(mFailedWriter) {
         // write failed data
-        char* fdata = new char[failedOut.size()];
-        memcpy(fdata, failedOut.c_str(), failedOut.size());
-        mFailedWriter->input(fdata, failedOut.size());
+        mFailedWriter->input(tid, failedOut);
+        failedOut = NULL;
     }
-    if(!mOptions->split.enabled)
-        mOutputMtx.unlock();
 
     if(mOptions->split.byFileLines)
         config->markProcessed(readPassed);
     else
         config->markProcessed(pack->count);
 
+    if(outstr)
+        delete outstr;
+    if(failedOut)
+        delete failedOut;
+
     delete pack->data;
     delete pack;
 
+    mPackProcessedCounter++;
+
     return true;
 }
 
-void SingleEndProcessor::initPackRepository() {
-    mRepo.packBuffer = new ReadPack*[PACK_NUM_LIMIT];
-    memset(mRepo.packBuffer, 0, sizeof(ReadPack*)*PACK_NUM_LIMIT);
-    mRepo.writePos = 0;
-    mRepo.readPos = 0;
-    //mRepo.readCounter = 0;
-    
-}
-
-void SingleEndProcessor::destroyPackRepository() {
-    delete mRepo.packBuffer;
-    mRepo.packBuffer = NULL;
-}
-
-void SingleEndProcessor::producePack(ReadPack* pack){
-    //std::unique_lock<std::mutex> lock(mRepo.mtx);
-    /*while(((mRepo.writePos + 1) % PACK_NUM_LIMIT)
-        == mRepo.readPos) {
-        //mRepo.repoNotFull.wait(lock);
-    }*/
-
-    mRepo.packBuffer[mRepo.writePos] = pack;
-    mRepo.writePos++;
-
-    /*if (mRepo.writePos == PACK_NUM_LIMIT)
-        mRepo.writePos = 0;*/
-
-    //mRepo.repoNotEmpty.notify_all();
-    //lock.unlock();
-}
-
-void SingleEndProcessor::consumePack(ThreadConfig* config){
-    ReadPack* data;
-    //std::unique_lock<std::mutex> lock(mRepo.mtx);
-    // buffer is empty, just wait here.
-    /*while(mRepo.writePos % PACK_NUM_LIMIT == mRepo.readPos % PACK_NUM_LIMIT) {
-        if(mProduceFinished){
-            //lock.unlock();
-            return;
-        }
-        //mRepo.repoNotEmpty.wait(lock);
-    }*/
-
-    mInputMtx.lock();
-    while(mRepo.writePos <= mRepo.readPos) {
-        usleep(1000);
-        if(mProduceFinished) {
-            mInputMtx.unlock();
-            return;
-        }
-    }
-    data = mRepo.packBuffer[mRepo.readPos];
-    mRepo.readPos++;
-
-    /*if (mRepo.readPos >= PACK_NUM_LIMIT)
-        mRepo.readPos = 0;*/
-    mInputMtx.unlock();
-
-    //lock.unlock();
-    //mRepo.repoNotFull.notify_all();
-
-    processSingleEnd(data, config);
-
-}
-
-void SingleEndProcessor::producerTask()
+void SingleEndProcessor::readerTask()
 {
     if(mOptions->verbose)
         loginfo("start to load data");
@@ -381,17 +327,18 @@ void SingleEndProcessor::producerTask()
     Read** data = new Read*[PACK_SIZE];
     memset(data, 0, sizeof(Read*)*PACK_SIZE);
     FastqReader reader(mOptions->in1, true, mOptions->phred64);
+    reader.setReadPool(mReadPool);
     int count=0;
     bool needToBreak = false;
     while(true){
         Read* read = reader.read();
-        // TODO: put needToBreak here is just a WAR for resolve some unidentified dead lock issue 
         if(!read || needToBreak){
             // the last pack
             ReadPack* pack = new ReadPack;
             pack->data = data;
             pack->count = count;
-            producePack(pack);
+            mInputLists[mPackReadCounter % mOptions->thread]->produce(pack);
+            mPackReadCounter++;
             data = NULL;
             if(read) {
                 delete read;
@@ -415,18 +362,19 @@ void SingleEndProcessor::producerTask()
             ReadPack* pack = new ReadPack;
             pack->data = data;
             pack->count = count;
-            producePack(pack);
+            mInputLists[mPackReadCounter % mOptions->thread]->produce(pack);
+            mPackReadCounter++;
             //re-initialize data for next pack
             data = new Read*[PACK_SIZE];
             memset(data, 0, sizeof(Read*)*PACK_SIZE);
-            // if the consumer is far behind this producer, sleep and wait to limit memory usage
-            while(mRepo.writePos - mRepo.readPos > PACK_IN_MEM_LIMIT){
+            // if the processor is far behind this reader, sleep and wait to limit memory usage
+            while( mPackReadCounter - mPackProcessedCounter > PACK_IN_MEM_LIMIT){
                 //cerr<<"sleep"<<endl;
                 slept++;
                 usleep(100);
             }
             readNum += count;
-            // if the writer threads are far behind this producer, sleep and wait
+            // if the writer threads are far behind this reader, sleep and wait
             // check this only when necessary
             if(readNum % (PACK_SIZE * PACK_IN_MEM_LIMIT) == 0 && mLeftWriter) {
                 while(mLeftWriter->bufferLength() > PACK_IN_MEM_LIMIT) {
@@ -453,10 +401,14 @@ void SingleEndProcessor::producerTask()
         }
     }
 
+    for(int t=0; t<mOptions->thread; t++)
+        mInputLists[t]->setProducerFinished();
+
     //std::unique_lock<std::mutex> lock(mRepo.readCounterMtx);
-    mProduceFinished = true;
-    if(mOptions->verbose)
-        loginfo("all reads loaded, start to monitor thread status");
+    mReaderFinished = true;
+    if(mOptions->verbose) {
+        loginfo("Loading completed with " + to_string(mPackReadCounter) + " packs");
+    }
     //lock.unlock();
 
     // if the last data initialized is not used, free it
@@ -464,41 +416,32 @@ void SingleEndProcessor::producerTask()
         delete[] data;
 }
 
-void SingleEndProcessor::consumerTask(ThreadConfig* config)
+void SingleEndProcessor::processorTask(ThreadConfig* config)
 {
+    SingleProducerSingleConsumerList<ReadPack*>* input = config->getLeftInput();
     while(true) {
         if(config->canBeStopped()){
-            mFinishedThreads++;
             break;
         }
-        while(mRepo.writePos <= mRepo.readPos) {
-            if(mProduceFinished)
+        while(input->canBeConsumed()) {
+            ReadPack* data = input->consume();
+            processSingleEnd(data, config);
+        }
+        if(input->isProducerFinished()) {
+            if(!input->canBeConsumed()) {
+                if(mOptions->verbose) {
+                    string msg = "thread " + to_string(config->getThreadId() + 1) + " data processing completed";
+                    loginfo(msg);
+                }
                 break;
-            usleep(1000);
-        }
-        //std::unique_lock<std::mutex> lock(mRepo.readCounterMtx);
-        if(mProduceFinished && mRepo.writePos == mRepo.readPos){
-            mFinishedThreads++;
-            if(mOptions->verbose) {
-                string msg = "thread " + to_string(config->getThreadId() + 1) + " data processing completed";
-                loginfo(msg);
             }
-            //lock.unlock();
-            break;
-        }
-        if(mProduceFinished){
-            if(mOptions->verbose) {
-                string msg = "thread " + to_string(config->getThreadId() + 1) + " is processing the " + to_string(mRepo.readPos) + " / " + to_string(mRepo.writePos) + " pack";
-                loginfo(msg);
-            }
-            consumePack(config);
-            //lock.unlock();
         } else {
-            //lock.unlock();
-            consumePack(config);
+            usleep(100);
         }
     }
+    input->setConsumerFinished();        
 
+    mFinishedThreads++;
     if(mFinishedThreads == mOptions->thread) {
         if(mLeftWriter)
             mLeftWriter->setInputCompleted();
@@ -512,7 +455,7 @@ void SingleEndProcessor::consumerTask(ThreadConfig* config)
     }
 }
 
-void SingleEndProcessor::writeTask(WriterThread* config)
+void SingleEndProcessor::writerTask(WriterThread* config)
 {
     while(true) {
         if(config->isCompleted()){
