@@ -11,7 +11,8 @@ using namespace std;
 class Sequence{
 public:
     Sequence();
-    Sequence(string seq);
+    Sequence(string*  seq);
+    ~Sequence();
     void print();
     int length();
     Sequence reverseComplement();
@@ -19,9 +20,10 @@ public:
     Sequence operator~();
 
     static bool test();
+    static string reverseComplement(string* origin);
 
 public:
-    string mStr;
+    string*  mStr;
 };
 
 #endif
\ No newline at end of file
